/// <reference path="v8debugger.d.ts" />
/// <reference path="DebuggerSupport.ts" />

"use strict";

module $_JbV8DebuggerSupport {
  import Request = v8debug.Request;
  import DebugCommandProcessor = v8debug.DebugCommandProcessor;

  var debuggerSupport = new DebuggerSupport();

  //noinspection JSUnresolvedVariable
  DebugCommandProcessor.prototype.super$processDebugJSONRequest = DebugCommandProcessor.prototype.processDebugJSONRequest;
  //noinspection JSUnresolvedVariable
  DebugCommandProcessor.prototype.processDebugJSONRequest = function (jsonRequest:string):string {
    var debugCommandProcessor:DebugCommandProcessor = <DebugCommandProcessor>this;
    var request:Request;
    try {
      request = JSON.parse(jsonRequest);
    } catch (e) {
      //noinspection JSUnresolvedVariable
      return '{"success": false, "message": ' + JSON.stringify('Cannot parse incoming message:' + jsonRequest + "\n\n" + e.stack) + '}';
    }

    try {
      if (request.command == null) {
        throw new Error('Command not specified');
      }

      var result:string;
      //noinspection FallThroughInSwitchStatementJS
      switch (request.command) {
        case 'getScopeObject':
          result = debuggerSupport.getScopeObject(request.arguments, debugCommandProcessor.exec_state_);
          break;

        case 'getFrames':
          result = debuggerSupport.getFrames(request.arguments, debugCommandProcessor.exec_state_);
          break;

        case 'getFrameReceiver':
          result = debuggerSupport.getFrameReceiver(request.arguments, debugCommandProcessor.exec_state_);
          break;

        case 'getObjects':
          result = debuggerSupport.getObjects(request.arguments);
          break;

        case 'getProperties':
          result = debuggerSupport.getProperties(request.arguments);
          break;

        case 'getIndexedProperties':
          result = debuggerSupport.getIndexedProperties(request.arguments);
          break;

        case 'getFunction':
          result = debuggerSupport.getFunction(request.arguments);
          break;

        case 'evaluate':
          result = debuggerSupport.evaluate(request.arguments, debugCommandProcessor.exec_state_);
          break;

        case 'continue':
        case 'disconnect':
          cacheManager.clearCache();
          // fall through

        default:
          return debugCommandProcessor.super$processDebugJSONRequest(jsonRequest);
      }

      return '{"request_seq": ' + request.seq +
      ', "success": true' +
      ', "running": ' + debugCommandProcessor.running_ +
      ', "body": ' + result +
      '}';
    }
    catch (e) {
      //noinspection JSUnresolvedVariable
      return '{"request_seq": ' + request.seq +
      ', "success": false' +
      ', "running": ' + debugCommandProcessor.running_ +
      ', "message":' + JSON.stringify(e.stack || e.toString()) +
      '}';
    }
  };
}