/*
 * Copyright 2000-2015 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var profiler = require("{v8-path}");
var sys = require("sys");
var http = require("http");
var url = require("url");
var fs = require("fs");

var pattern = 'filePath=';
var cnt = 0;
var cpuCnt = 0;
var isCpuProfiling = false;

function isDelimited(filePath, encodedChar) {
  return filePath.indexOf(encodedChar) == 0 && filePath.indexOf(encodedChar, filePath.length - 1 - encodedChar.length) > 0;
}

var server = null;

function saveSnapshot(cnt, filePath) {
  var wstream = null;
  try {
    var snapshot = profiler.takeSnapshot("snapshot" + (cnt++));
    var options = {encoding: 'utf8'};
    wstream = fs.createWriteStream(filePath, options);
    snapshot.serialize(function iterator(data, length) {
      wstream.write(data);
    }, function callback() {
    });
    return true;
  } catch (e) {
    console.error(e);
    return false;
  } finally {
    if (wstream != null) {
      wstream.end();
    }
  }
}

function startProfiling(cpuCnt) {
  profiler.startProfiling("CpuSnaphot" + cpuCnt);
}

function stopProfiling(cpuCnt, filePath) {
  var wstream = null;
  try {
    var options = {encoding: 'utf8'};
    wstream = fs.createWriteStream(filePath, options);
    var profile = profiler.stopProfiling();
    wstream.write(JSON.stringify(profile));
  } catch (e) {
    //
  } finally {
    if (wstream != null) {
      wstream.end();
    }
  }
  /*  Handle<String> GetTitle() const;

  const CpuProfileNode* GetTopDownRoot() const;

  int GetSamplesCount() const;

  const CpuProfileNode* GetSample(int index) const;
  int64_t GetSampleTimestamp(int index) const;

  int64_t GetStartTime() const;

  int64_t GetEndTime() const;
*/
}

function process(request,response) {
  var parse = url.parse(request.url);
  var path = parse.pathname;
  var filePath = parse.query && parse.query.length > 0 && parse.query.indexOf(pattern) == 0 ?
                 parse.query.substring(pattern.length) : null;

  filePath = decodeURIComponent(filePath);
  if (filePath != null) {
    if (isDelimited(filePath, "'") || isDelimited(filePath, "\"")) {
      filePath = filePath.substring(1, filePath.length - 1);
    }
  }
  if (filePath != null && "/takeHeapSnapshot/" == path) {
    if (saveSnapshot(cnt, filePath)) {
      response.write("ok");
    }
  } else if ("/startCpuProfiling" == path) {
    if (!isCpuProfiling) {
      startProfiling(cpuCnt++);
      isCpuProfiling = true;
      response.write("ok");
    } else {
      response.write("wrong_state");
    }
  } else if (filePath != null && "/stopCpuProfiling/" == path) {
    if (isCpuProfiling) {
      stopProfiling(cpuCnt, filePath);
      isCpuProfiling = false;
      response.write("ok");
    } else {
      response.write("wrong_state");
    }
  } else if ("/stop" == path) {
    response.writeHeader(200, {"Content-Type": "text/plain"});
    if (server != null) {
      server.close();
    }
    response.write("ok");
  } else {
    response.writeHeader(404, {"Content-Type": "text/plain"});
  }
  response.end();
}

server = http.createServer(function (request,response){
  try {
    process(request, response);
  } catch (e) {
    console.error(e);
  }
}).listen({port});

var a = require("{app-path}");
